// This file is part of cwait, copyright (c) 2015-2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
var Task_1 = require('./Task');
var TaskQueue = (function () {
    function TaskQueue(Promise, concurrency) {
        var _this = this;
        this.backlog = [];
        this.busyCount = 0;
        this.Promise = Promise;
        this.concurrency = concurrency;
        this.nextBound = function () { return _this.next(); };
    }
    /** Add a new task to the queue.
      * It will start when the number of other concurrent tasks is low enough. */
    TaskQueue.prototype.add = function (func) {
        if (this.busyCount < this.concurrency) {
            // Start the task immediately.
            ++this.busyCount;
            var result = func();
            result.then(this.nextBound, this.nextBound);
            return (result);
        }
        else {
            // Schedule the task and return a promise resolving
            // to the result of task.start().
            var task = new Task_1.Task(func);
            this.backlog.push(task);
            return (task.delay(this.Promise));
        }
    };
    /** Wrap a function returning a promise, so that before running
      * it waits until concurrent invocations are below this queue's limit. */
    TaskQueue.prototype.wrap = function (func, thisObject) {
        var _this = this;
        return (function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            return _this.add(function () { return func.apply(thisObject, args); });
        });
    };
    /** Start the next task from the backlog. */
    TaskQueue.prototype.next = function () {
        var task = this.backlog.shift();
        if (task)
            task.resume(this.nextBound);
        else
            --this.busyCount;
    };
    return TaskQueue;
}());
exports.TaskQueue = TaskQueue;
